# Deleting Objects
## Question
When I delete an object explicitly, I assume I am deleting the address of the object.  Thus:
```cpp
class A {.... }
```

Create an instance of A:
```cpp
A *a = new A();
```

to delete a, I would execute delete a?

## Reply
When you create an instance of a class `A` using dynamic memory allocation with `new`, like this:

```cpp
A *a = new A();
```

The pointer `a` holds the address of the dynamically allocated object of type `A`. To delete this object and free the memory, you would use:

```cpp
delete a;
```

This call to `delete` does the following:
1. **Invokes the destructor** of the object `a` (if one is defined).
2. **Frees the memory** that was allocated for the object `a`.

### Important Notes:
- **Deleting a pointer** doesn't delete the pointer itself, it deletes the object (or memory) that the pointer points to. After calling `delete a`, the pointer `a` will still hold the now-invalid address, so it’s a good practice to set it to `nullptr` after deletion:
  ```cpp
  delete a;
  a = nullptr;
  ```

- If `a` points to an array of objects allocated with `new[]`, you need to use `delete[]` instead:
  ```cpp
  A *a = new A[10]; // Allocating an array of 10 objects
  delete[] a;       // Deleting the array of objects
  ```

Using `delete[]` ensures that the destructors for each object in the array are called.
