// Created by GPT4o-mini
// Question: Update @main.cpp to provide a range of examples that illustrate the uses of smart pointers.

#include <iostream>
#include <memory>
#include <vector>

void exampleUniquePointer() {
    // Create a unique pointer
    // Unique pointer occupies 16 bytes
    std::unique_ptr<int> uniquePtr1 = std::make_unique<int>(10);
    std::unique_ptr<int> uniquePtr2(new int(10));

    std::cout << "Unique Pointer 1 Value: " << *uniquePtr1 << std::endl; // {{ edit_1 }}
    std::cout << "Unique Pointer 1 Address: " << uniquePtr1 << std::endl; // {{ edit_1 }}
    std::cout << "Unique Pointer 2 Value: " << *uniquePtr2 << std::endl; // {{ edit_1 }}
    std::cout << "Unique Pointer 2 Address: " << uniquePtr2 << std::endl; // {{ edit_1 }}
}

void exampleSharedPointer() {
    // Create a shared pointer
    std::shared_ptr<int> sharedPtr1 = std::make_shared<int>(20);
    std::shared_ptr<int> sharedPtr2 = sharedPtr1; // Shared ownership
    std::shared_ptr<int> sharedPtr3(new int(10)); // not recommended
    std::shared_ptr<int> sharedPtr4 = sharedPtr3; // Shared ownership
    std::cout << "Shared Pointer Value: " << *sharedPtr1 << ", Count: " << sharedPtr1.use_count() << std::endl; // {{ edit_2 }}
}

void exampleWeakPointer() {
    // Create a weak pointer
    std::shared_ptr<int> sharedPtr = std::make_shared<int>(30);
    std::weak_ptr<int> weakPtr = sharedPtr; // Weak reference
    if (auto lockedPtr = weakPtr.lock()) {
        std::cout << "Weak Pointer Value: " << *lockedPtr << std::endl; // {{ edit_3 }}
    } else {
        std::cout << "Weak Pointer is expired." << std::endl;
    }
}

int main() {
    std::cout << std::endl << "Example Unique Pointer" << std::endl;
    exampleUniquePointer();

    std::cout << std::endl << "Example Shared Pointer" << std::endl;
    exampleSharedPointer();

    std::cout << std::endl << "Example Weak Pointer" << std::endl;
    exampleWeakPointer();

    return 0;
}
