// Created by Sonnet 3.5
// Question: Update @main.cpp to provide a range of examples that illustrate the uses of smart pointers.

#include <iostream>
#include <memory>
#include <vector>
using namespace std;

class Resource {
public:
    Resource(int value) : data(value) {
        cout << "Resource constructed with value " << data << endl;
    }
    ~Resource() {
        cout << "Resource destroyed with value " << data << endl;
    }
    int getData() const { return data; }
private:
    int data;
};

// ... existing code ...

void example1()
{
    cout << endl << "Example 1: unique_ptr" << endl;
    unique_ptr<Resource> uniqueRes = make_unique<Resource>(42);
    cout << "Unique resource value: " << uniqueRes->getData() << endl;
}


int main() {
    // Example 1: unique_ptr
    cout << endl << "Example 1: unique_ptr" << endl;
    unique_ptr<Resource> uniqueRes = make_unique<Resource>(42);
    cout << "Unique resource value: " << uniqueRes->getData() << endl;

    // Example 2: shared_ptr
    cout << endl << "Example 2: shared_ptr" << endl;
    shared_ptr<Resource> sharedRes1 = make_shared<Resource>(10);
    {
        auto sharedRes2 = sharedRes1;
        cout << "Shared count: " << sharedRes1.use_count() << endl;
    }
    cout << "Shared count after scope: " << sharedRes1.use_count() << endl;

    // Example 3: weak_ptr
    cout << endl << "Example 3: weak_ptr" << endl;
    weak_ptr<Resource> weakRes = sharedRes1;
    if (auto lockedRes = weakRes.lock()) {
        cout << "Weak resource value: " << lockedRes->getData() << endl;
    }

    // Example 4: unique_ptr with custom deleter
    cout << endl << "Example 4: unique_ptr with custom deleter" << endl;
    auto customDeleter = [](Resource* r) {
        cout << "Custom deleter called" << endl;
        delete r;
    };
    unique_ptr<Resource, decltype(customDeleter)> customRes(new Resource(99), customDeleter);

    // Example 5: shared_ptr in a container
    cout << endl << "Example 5: shared_ptr in a container" << endl;
    vector<shared_ptr<Resource>> resourceVector;
    resourceVector.push_back(make_shared<Resource>(1));
    resourceVector.push_back(make_shared<Resource>(2));
    resourceVector.push_back(make_shared<Resource>(3));

    // ... existing code ...

    return 0;
}
